from pprint import pprint
from pathlib import Path
from tqdm import tqdm
#from tqdm.contrib.concurrent import process_map

import numpy as np
import pandas as pd

import dolphindb as ddb
import dolphindb.settings as keys

import sqlalchemy as sa

from .DDBLoader import DDBLoader


class DDBBasicInfoLoader(DDBLoader):

    ddb_path = "dfs://info_stock_ts"
    ddb_dbname = "ddb_info_stock_ts"


    # 这些映射表似乎没有什么用，每张表手动做一下表结构和插入动作就可以了

    table_name_mapping = {
        'IndustryInfo' : 'info_industry',
        'ListAndDelist' : 'info_list_delist'
    }

    date_col_set = {
        'EnterDate',
        'ExitDate'
        'ListDate',
        'DelistDate'
    }

    ddb_type_mapping = {
        'float' : 'DOUBLE',
        'int' : 'INT',
        'text' : 'STRING',
        'varchar' : 'STRING',
        'str' : 'STRING'
    }

    # 基本面数据库现在存放在91服务器之上
    mssql_config = {
        'host' : '192.168.1.7',
        'username' : 'sa',
        'password' : 'passw0rd!',
        'dbname' : 'StockInfo'
    }


    def __init__(self):
        super().__init__()
        # 重新设定mssql_engine对象，此时我们需要使用基本面数据库
        self.mssql_engine = sa.create_engine(
            "mssql+pyodbc://{username}:{password}@{host}/{dbname}?driver=ODBC+Driver+18+for+SQL+Server".format(**self.mssql_config),
            connect_args = {
                "TrustServerCertificate": "yes"
            }, echo=False
        )



    def create_ddb_database(self):
        self.ddb_sess.run("""
            {dbName} = database(
                directory = '{dbPath}',
                partitionType = HASH,
                partitionScheme = [SYMBOL, 10],
                engine = 'TSDB'
            )
        """.format(
            dbName = self.ddb_dbname,
            dbPath = self.ddb_path,
        ))


    def _create_info_industry_table(self):
        with self.mssql_engine.connect() as conn:
            stat = "select * from IndustryInfo"
            row_list = list(conn.execute(stat).fetchall())

        capacity = len(row_list)
        df = pd.DataFrame(row_list)
        df['StockID'] = self.tscode_to_windcode(df['StockID'])
        df['EnterDate'] = self.make_date(df['EnterDate'])
        df['ExitDate'] = self.make_date(df['ExitDate'])

        self.ddb_sess.run(f"""
            tbl = table(
                {capacity}:0, 
                `code`industry_code`industry_name`industry_level`enter_date`exit_date`is_new`industry_class,
                [SYMBOL, SYMBOL, SYMBOL, INT, DATE, DATE, INT, SYMBOL]
            );
        """)

        self.ddb_sess.run("tableInsert{tbl}", df)
        self.ddb_sess.run(f"""
            dropTable({self.ddb_dbname}, "info_industry");

            info_industry = createPartitionedTable(
                dbHandle = {self.ddb_dbname},
                table = tbl,
                tableName = "info_industry",
                partitionColumns='code',
                compressMethods = {{'enter_date' : 'delta', 'exit_date' : 'delta'}},
                sortColumns = ['code', 'enter_date', 'exit_date']
            );

            info_industry.tableInsert(tbl);
            """)


    def _create_info_list_delist_table(self):
        with self.mssql_engine.connect() as conn:
            stat = "select * from ListAndDelist"
            row_list = list(conn.execute(stat).fetchall())

        capacity = len(row_list)
        df = pd.DataFrame(row_list)
        df['StockID'] = self.tscode_to_windcode(df['StockID'])
        df['ListDate'] = self.make_date(df['ListDate'])
        df['DelistDate'] = self.make_date(df['DelistDate'])

        self.ddb_sess.run(f"""
            tbl = table(
                {capacity}:0, 
                `code`list_date`delist_date,
                [SYMBOL, DATE, DATE]
            );
        """)

        self.ddb_sess.run("tableInsert{tbl}", df)
        self.ddb_sess.run(f"""
            dropTable({self.ddb_dbname}, "info_list_delist");

            info_list_delist = createPartitionedTable(
                dbHandle = {self.ddb_dbname},
                table = tbl,
                tableName = "info_list_delist",
                partitionColumns='code',
                compressMethods = {{'list_date' : 'delta', 'delist_date' : 'delta'}},
                sortColumns = ['code', 'list_date', 'delist_date']
            );
            
            info_list_delist.tableInsert(tbl);
            """)


    def create_ddb_partition_table(self):
        """创建分区表"""
        self._create_info_industry_table()
        self._create_info_list_delist_table()




